// -------------------------------------------------------------------
// Defines.java:
//    This class takes care of recording the definitions in a stack
//    implementation so that we can search through the stacks and find
//    the definition of the correct scope.  It also allows us to free a
//    set of defines when a file goes out of scope.
// -------------------------------------------------------------------
import java.util.*;

public class Defines {
   Vector vDefStack;
   Date startDate;

   // constructor, plain
   public Defines() {
      startDate = new Date();
      vDefStack = new Vector();
   }

   public void push() {
      // this method will put another property object on the stack
      Hashtable temp;

      temp = new Hashtable();
      vDefStack.insertElementAt(temp, 0);
   }

   public void pop() {
      // this method will remove the most recent property from the stack
      if (vDefStack.size() > 0) {
         vDefStack.removeElementAt(0);
      }
   }

   // return true if the keyword has been defined, false otherwise
   public boolean isKeywordDefined(String keyword) {
      Object retVal = getValue(keyword);
      if (retVal == null)
         return false;
      else
         return true;
   }

   public Object getValue(String keyword) {
      // this method will go through the properties and will return the
      // value that matches the keyword, or NULL

      // cycle through the property objects until we find the keyword, if not
      // found, try reserved, if still not found will return null
      boolean bFound=false;
      Hashtable temp;
      Object retVal=null;
      String tKey=null;

      Enumeration vecEnum = vDefStack.elements();
      while (vecEnum.hasMoreElements() && !bFound) {
         temp = (Hashtable) vecEnum.nextElement();
         tKey = keyword.toUpperCase();
         if (temp.containsKey(tKey)) {
            retVal = temp.get(tKey);
            bFound = true;
         }
      }

      if (bFound)
         return retVal;

      // if we didn't find it in the lists, maybe it is a reserved keyword
      return getReserved(keyword);
   }

   public void addKeyword(String keyword, Object value) {
      // this method will add a keyword/value pair to the current property object
      if (vDefStack.size() > 0) {
         Hashtable temp;

         temp = (Hashtable) vDefStack.elementAt(0);
         temp.put(keyword.toUpperCase(), value);
      }
   }

   public void removeKeyword(String keyword) {
      // this method will remove a keyword/value pair from the current property object
      if (vDefStack.size() > 0) {
         Hashtable temp;

         temp = (Hashtable) vDefStack.elementAt(0);
         temp.remove(keyword.toUpperCase());
      }
   }

   private Object getReserved(String keyword) {
      // this method will return either the value for a reserved keyword
      // or the NULL string if the keyword is not a reserved word
      Object retVal=null;
      Date curDate;
      int pos;
      String temp;

      // we might as well get the current date
      curDate = new Date();

      if (keyword.equalsIgnoreCase("#CURDATE")) {
         temp = curDate.toLocaleString();
         pos =  temp.indexOf(" ");
         if (pos > 0)
            retVal = temp.substring(0, pos);
         else
            retVal = temp;
      }
      if (keyword.equalsIgnoreCase("#STARTDATE")) {
         temp = startDate.toLocaleString();
         pos =  temp.indexOf(" ");
         if (pos > 0)
            retVal = temp.substring(0, pos);
         else
            retVal = temp;
      }
      if (keyword.equalsIgnoreCase("#CURYR4")) {
         int work = curDate.getYear();
         if (work < 50)
            work += 2000;
         else
            work += 1900;
         Integer i = new Integer(work);
         retVal = i.toString();
      }
      if (keyword.equalsIgnoreCase("#CURYR2")) {
         Integer i = new Integer(curDate.getYear());
         retVal = i.toString();
      }
      else if (keyword.equalsIgnoreCase("Time")) {
         retVal = new String("RESERVED TIME - NYI");
      }
      else if (keyword.equalsIgnoreCase("FileDate")) {
         retVal = new String("RESERVED FILEDATE - NYI");
      }
      else if (keyword.equalsIgnoreCase("FileName")) {
         retVal = new String("RESERVED FILENAME - NYI");
      }

      return retVal;
   }
}
